import { forwardRef, useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { mergeRefs } from 'react-merge-refs';

import {
    BaseSelect,
    BaseSelectChangePayload,
    Optgroup as DefaultOptgroup,
    Option as DefaultOption,
    OptionsList as DefaultOptionsList,
    GroupShape,
    OptionShape,
} from '@controls/future/Select';
import { Arrow as DefaultArrow } from '@controls/future/Select/components/arrow';

import { colors } from '@scripts/gds';

import { InputProps } from '../Input';
import { TagList } from '../SelectWithTags/TagList';
import AutocompleteField from './AutocompleteField';
import Clear from './Clear';
import { AutocompleteProps } from './types';

export const Autocomplete = forwardRef<HTMLInputElement, AutocompleteProps>(
    (
        {
            Arrow = DefaultArrow,
            OptionsList = DefaultOptionsList,
            Optgroup = DefaultOptgroup,
            Option = DefaultOption,
            Input,
            inputProps = {},
            onInput,
            value,
            readOnly,
            multiple,
            closeOnSelect = !multiple,
            options = [],
            fieldProps,
            ...restProps
        },
        ref
    ) => {
        const props = useMemo(
            () => ({
                ref,
                autocomplete: true,
                options,
                closeOnSelect,
                Option,
                Arrow,
                multiple,
                Field: multiple ? TagList : AutocompleteField,
                fieldProps: {
                    Input,
                    onInput,
                    value,
                    inputProps,
                    readOnly,
                    ...fieldProps,
                },
                Optgroup,
                OptionsList,
                ...restProps,
            }),
            [
                multiple,
                Arrow,
                Input,
                Optgroup,
                Option,
                OptionsList,
                closeOnSelect,
                fieldProps,
                inputProps,
                onInput,
                options,
                readOnly,
                ref,
                restProps,
                value,
            ]
        );

        return <BaseSelect emitChangeOnClick {...props} />;
    }
);

Autocomplete.displayName = 'Autocomplete';

/**
 * Фичи:
 * 1) если кликнуть на опцию, она выберется. при этом текстовый инпут примет значение опции
 * 2) если извне изменится значение, то текстовый инпут примет значение key соответствующего value
 * 3) если пользователь закроет выпадающий список, при этом не выбрав значение, то инпут примет последнее key
 */

export const useAutocomplete = (
    initialSelectedValues: any[],
    multiple: boolean,
    options: AutocompleteProps['options'],
    reinitialize = true,
    showSelected = true
) => {
    const [selectedValues, setSelectedValues] = useState(initialSelectedValues);
    const selectedOptions = useMemo(
        () =>
            options.filter(e => {
                if ('value' in e) return selectedValues.includes(e.value);
                return false;
            }) as OptionShape[],
        [options, selectedValues]
    );

    const currentSelectedOptions = useRef(selectedOptions);
    currentSelectedOptions.current = selectedOptions;

    const [search, setSearch] = useState('');

    const getKeyByValue = useCallback(
        (value: any) => {
            const result = options.find(e => {
                if ('value' in e) return e.value === value;
                return false;
            });

            if (!result || !('key' in result)) return null;

            return result.key;
        },
        [options]
    );

    // если извне изменится значение, то текстовый инпут примет значение key соответствующего value
    useEffect(() => {
        if (!reinitialize) return;

        console.log('initialSelectedValues changed to=', initialSelectedValues);

        if (!multiple && initialSelectedValues.length > 0) {
            const key = getKeyByValue(initialSelectedValues[0]);

            if (key !== null) {
                console.log('external. setting search to', key);
                setSearch(key);
            }
        }

        setSelectedValues(initialSelectedValues);
    }, [initialSelectedValues, multiple, getKeyByValue, reinitialize]);

    const matchOption = useCallback((option: GroupShape | OptionShape, inputValue: string) => {
        if (!inputValue) return true;

        if ('key' in option) {
            return option.key.toLowerCase().includes(inputValue.toLowerCase());
        }

        if ('label' in option) {
            return option.label?.toLowerCase().includes(inputValue.toLowerCase());
        }

        return false;
    }, []);

    const [isDirtySearch, setDirtySearch] = useState(false);

    const handleInput = useCallback<Exclude<InputProps['onChange'], undefined>>((_, payload) => {
        setSearch(payload.value);
        setDirtySearch(!!payload.value.length);
    }, []);

    const handleChange = ({ selected }: BaseSelectChangePayload) => {
        setTimeout(() => {
            if (multiple) {
                setSearch('');
            } else {
                console.log('internal. setting search to', selected ? selected.key : '');

                setSearch(selected ? selected.key : '');
            }
        }, 0);

        if (selected) {
            const vals = new Set(selectedValues);
            vals.add(selected.value);
            setSelectedValues([...vals.values()]);
        }
    };

    const filteredOptions = useMemo(
        () =>
            !isDirtySearch
                ? options
                : options.filter(option => {
                      if (!('value' in option)) return false;
                      const isSelected = selectedValues.includes(option.value);
                      const isMatched = matchOption(option, search);

                      return (!isSelected || showSelected) && isMatched;
                  }),
        [isDirtySearch, options, selectedValues, matchOption, search, showSelected]
    );

    const onClose = useCallback(() => {
        setDirtySearch(false);

        setTimeout(() => {
            if (!multiple) {
                setSearch(currentSelectedOptions.current?.[0]?.key || '');
            } else {
                setSearch('');
            }
        }, 0);
    }, [multiple]);

    return {
        value: search,
        setValue: setSearch,
        filteredOptions,
        handleInput,
        selectedValues,
        handleChange,
        matchOption,
        selectedOptions,
        onClose,
    };
};

const FormikAutocomplete = forwardRef<
    HTMLInputElement,
    AutocompleteProps & {
        field?: { value: any; onChange: (val: any) => void };
        meta?: any;
        helpers?: { setValue: (value: any) => void };
    }
>(
    (
        {
            multiple = false,
            meta,
            field,
            helpers,
            onOpen,
            onChange,
            onInput,
            onBlur,
            options,
            fieldProps,
            withTags = multiple,
            placeholderSelected,
            collapseTagList = false,
            moveInputToNewLine = true,
            transformCollapsedTagText,
            transformTagText,
            ...props
        },
        ref
    ) => {
        const initialSelectedValues = useMemo(() => {
            if (multiple) return Array.isArray(field?.value) ? field?.value || [] : [];

            return field?.value === null ? [] : [field?.value];
        }, [field?.value, multiple]);

        const { value, setValue, handleInput, onClose, handleChange, selectedOptions, filteredOptions } =
            useAutocomplete(initialSelectedValues, multiple, options);

        const inputRef = useRef<HTMLInputElement | null>(null);

        const selectedCount = selectedOptions.length;
        const hasValue = !!value;

        const showSelectedPlaceholder = !!placeholderSelected && !hasValue && selectedCount > 0;

        const isOpenRef = useRef(false);

        const hasSelected = selectedCount > 0;
        let canClear;

        if (!multiple) {
            canClear = hasValue;
        } else {
            canClear = hasValue || hasSelected;
        }

        if (props.hideClearButton) {
            canClear = false;
        }

        const isEverythingSelected = options && selectedCount >= options.length;

        const [isPopoverOpen, setPopoverOpen] = useState<boolean | undefined>(false);

        const handleDeleteTag = useCallback(
            (deletedKey: string) => {
                const tags = selectedOptions.filter(tag => {
                    const key = typeof tag === 'string' ? tag : tag.key;

                    return deletedKey !== key;
                });

                if (onChange) {
                    onChange({ selectedMultiple: tags, initiator: null, selected: tags?.[0] || null });
                }

                helpers?.setValue(tags.map(e => e.value));
            },
            [onChange, selectedOptions, helpers]
        );

        return (
            <Autocomplete
                ref={ref}
                {...(withTags && {
                    hideSelectedOptions: true,
                    showEmptyOptionsList: true,
                })}
                options={filteredOptions}
                {...props}
                value={value}
                selected={selectedOptions}
                onOpen={payload => {
                    setPopoverOpen(payload.open);

                    onOpen?.(payload);
                    if (typeof payload.open === 'boolean') {
                        isOpenRef.current = payload.open;
                    }

                    if (!payload.open) {
                        onClose();
                    }
                }}
                onInput={e => {
                    handleInput(e, {
                        value: e.currentTarget.value,
                    });

                    onInput?.(e);

                    if (!multiple && !e.currentTarget.value) {
                        field?.onChange({
                            target: {
                                value: null,
                            },
                        });
                    }
                }}
                onChange={payload => {
                    handleChange(payload);

                    if (typeof field?.onChange === 'function') {
                        const value = multiple ? payload.selectedMultiple.map(e => e.value) : payload.selected?.value;

                        console.log('setting value=', value);

                        field.onChange({
                            target: {
                                value,
                            },
                        });

                        onChange?.(payload);

                        if (!multiple) {
                            onClose();
                        }

                        return;
                    }

                    if (helpers) {
                        setTimeout(() => {
                            if (!multiple) {
                                helpers.setValue(payload.selected?.value);
                            } else {
                                helpers.setValue(payload.selectedMultiple.map(e => e.value));
                            }
                        }, 1);
                    }

                    onChange?.(payload);
                }}
                allowUnselect={multiple}
                multiple={multiple}
                onBlur={onBlur}
                inputProps={{
                    ref: mergeRefs([inputRef /* props.inputProps!.ref */]),
                    placeholder: props.placeholder,
                    ...(showSelectedPlaceholder && {
                        placeholder: `${placeholderSelected} ${selectedCount}`,
                    }),
                }}
                fieldCSS={{
                    ...(showSelectedPlaceholder && {
                        'input::placeholder': {
                            transition: 'color .2s ease',
                            willChange: 'color',
                        },
                        'input:not(:focus)::placeholder': {
                            color: colors.black,
                        },
                    }),
                }}
                fieldProps={{
                    error: meta?.error,
                    ...(withTags && {
                        handleDeleteTag,
                        collapseTagList,
                        moveInputToNewLine,
                        transformCollapsedTagText,
                        transformTagText,
                        value,
                        autocomplete: true,
                        isPopoverOpen,
                    }),
                    rightAddons: (
                        <Clear
                            visible={canClear && !props.disabled}
                            clear={() => {
                                setValue('');
                                if (!props.allowUnselect) {
                                    setTimeout(() => {
                                        inputRef.current?.focus();
                                    }, 10);
                                    return;
                                }
                                if (!helpers) return;

                                if (!multiple) {
                                    helpers.setValue(null);
                                } else {
                                    helpers.setValue([]);
                                }
                            }}
                            focus={() => {
                                if (isOpenRef.current) {
                                    inputRef.current?.focus();
                                } else {
                                    inputRef.current?.blur();
                                }
                            }}
                            css={{ marginLeft: 'auto' }}
                        />
                    ),
                    ...fieldProps,
                }}
                optionsListProps={{
                    emptyPlaceholder: (
                        <div style={{ display: 'flex', justifyContent: 'center' }}>
                            {isEverythingSelected ? 'Все элементы выбраны' : null}
                            {!isEverythingSelected && hasValue ? <p>Ничего не нашлось</p> : null}
                            {!isEverythingSelected && !hasValue ? <p>Начинайте вводить</p> : null}
                        </div>
                    ),
                }}
            />
        );
    }
);

FormikAutocomplete.displayName = 'FormikAutocomplete';

export default FormikAutocomplete;
